@@ -1,9 +1,12 @@
-from .discovery import SensemeDiscovery as SensemeDiscovery
+from .discovery import Discover as Discover
 from .discovery import Discover_Any as Discover_Any
+from .discovery import SensemeDiscovery as SensemeDiscovery
 from .fan import SensemeFan as SensemeFan
+from .version import __version__
 
 __all__ = [
+    "SensemeFan",
     "SensemeDiscovery",
     "Discover_Any",
-    "SensemeFan",
+    "Discover",
 ]
